# Copyright 2014 Microsoft Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import imp
import os
import sys

project_root = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
waagent = imp.load_source('waagent', os.path.join(project_root, 'waagent'))
sys.path.insert(0, project_root)

waagent.LoggerInit('/dev/stdout', '/dev/null')

